import pygame 
from pygame.locals import *

from Classes.Objects.Cards.MonsterCard import MonsterCard
from Classes.System.Window import Window
from Classes.System.InputHandler import InputHandler
from Classes.Objects.World import World

class App:

    __window:Window
    __running:bool = True
    __FPS = 60
    __clock = pygame.time.Clock()
    __myFont:pygame.font
    __world:World
    __inputHandler: InputHandler

    def __init__(self, width:int=1920, height:int=1080, title:str="default title"):
        pygame.font.init() 
        self.__myFont = pygame.font.SysFont('Comic Sans MS', 30)
        self.__window = Window(width=width, height=height, title=title)
        self.__inputHandler = InputHandler()
        
        # game word
        self.__world = World()

        self.startGameLoop()
        self.onCleanup()

    def startGameLoop(self):

        # create sprite groups
        cards = pygame.sprite.Group()

        testMonsterCard = MonsterCard((500,500), "Assets/Cards/MonsterCards/testmonstercard/")
        cards.add(testMonsterCard)

        while self.__running:
            self.__clock.tick(self.__FPS)

            self.__window.getScreen().fill((0,0,0))

            # render world
            self.__inputHandler.getMouseHover(self.__world)
            self.__window.drawWorld(self.__world)

            # update sprite groups
            cards.update()

            # draw groups
            self.__window.drawSpriteGroup(cards)

            # event handler
            self.handleEvent(pygame.event.get())

            # emits update to the game
            pygame.display.update()

    # handles incoming event queue
    def handleEvent(self, events):
        for event in events:
            if event.type == pygame.QUIT:
                self.onCleanup()
            if event.type == pygame.MOUSEBUTTONDOWN:
                pass
            if event.type == pygame.MOUSEBUTTONUP:
                pass

    # sets the running state for the gameloop
    def setRunning(self, running:bool):
        self.__running = running

    # ensures the gameloop stops running and the pygame instance is stopped properly
    def onCleanup(self):
        self.__running = False
        pygame.quit()